# self is a sample Python script.
import random
import traceback
import platform

# Press Shift+F10 to execute it or replace it with your code.
# Press Double Shift to search everywhere for classes, files, tool windows, actions, and settings.


import pika  # 安装pika模块连接mq
import json
import threading
import handleManager
import schedule
import calendar
import time
import operator
import os
from git.repo import Repo
from git.repo.fun import is_git_dir

thread_local_context = threading.local()


# rabbitmq 配置信息
class RabbitMQConfig(object):
    host = os.environ.get('RABBIT_MQ_HOST', "itlym.cn")
    port = eval(os.environ.get('RABBIT_MQ_PORT', '5672'))
    username = os.environ.get('RABBIT_MQ_USER', "spider")
    password = os.environ.get('RABBIT_MQ_PWD', "spider")
    vhost = os.environ.get('RABBIT_MQ_VHOST', "spider")
    customer_num = eval(os.environ.get('RABBIT_MQ_CUSTOMER_NUM', '5'))


# 本地仓库位置
local_path = os.environ.get('FAAS_CODE_DIR', "./spider_result_handler")
# 远程仓库位置
remote_path = os.environ.get('REMOVE_GIT_PATH', 'http://git.itlym.cn/gogs/dynamic_py_function.git')


######################################################################################################

class GitRepository(object):
    """
    git仓库管理
    """

    ts = 0

    def __init__(self, local_path, repo_url, branch='master'):
        self.local_path = local_path
        self.repo_url = repo_url
        self.repo = None
        self.initial(repo_url, branch)

    def initial(self, repo_url, branch):
        """
        初始化git仓库
        :param repo_url:
        :param branch:
        :return:
        """
        if not os.path.exists(self.local_path):
            os.makedirs(self.local_path)

        git_local_path = os.path.join(self.local_path, '.git')
        if not is_git_dir(git_local_path):
            self.repo = Repo.clone_from(repo_url, to_path=self.local_path, branch=branch)
        else:
            self.repo = Repo(self.local_path)

    def pull(self):
        """
        从线上拉最新代码
        :return:
        """
        return self.repo.git.pull()


class RabbitMQ(object):
    def __init__(self, username, password, host, port, virtual_host, exchange, routing_key, queue_name, exchange_type,
                 arguments=None):
        self.connect = None
        self.username = username
        self.password = password
        self.host = host
        self.port = port
        self.virtual_host = virtual_host
        self.exchange = exchange
        self.routing_key = routing_key
        self.queue_name = queue_name
        self.exchange_type = exchange_type
        self.arguments = arguments
        # 登录凭证
        self.credentials = pika.PlainCredentials(username=self.username, password=self.password)
        self.get_connect()  # 初始化RabbitMQ实例对象时完成连接rabbitmq服务器


    def get_connect(self):
        self.connect = pika.BlockingConnection(
            pika.ConnectionParameters(host=self.host, port=self.port, virtual_host=self.virtual_host,
                                      credentials=self.credentials))
        self.channel = self.connect.channel()  # 客户端连接rabbitmq服务端后开辟管道，每个channel代表一个会话任务

        
    def publishMsg(self, body, delay_ms=None):
        headers = {}
        if delay_ms:
            headers = {"x-delay": delay_ms}

        try:
            # 发消息
            self.channel.basic_publish(
                exchange=self.exchange,
                routing_key=self.routing_key,  # 根据exchange和routing进而指定 队列
                body=body,  # 发送的数据
                properties=pika.BasicProperties(delivery_mode=2, headers=headers)  # 消息持久化存到硬盘， 1是非持久化
            )
        except Exception as e:
            print('producer Error: ' + traceback.format_exc())
            self.connect.close()
            self.get_connect()
            # 创建exchange消息交换机， 并且exchange持久化
            self.channel.exchange_declare(exchange=self.exchange, exchange_type=self.exchange_type, durable=True,
                                          arguments=self.arguments)
            # 声明队列， 并且队列持久化
            self.channel.queue_declare(queue=self.queue_name, durable=True)
            # 通过routing_key 绑定 消息交换机和队列
            self.channel.queue_bind(self.queue_name, self.exchange, self.routing_key)
            pass


        

    def doConsumer(self, callback):
        while True:
            print('消费任务启动')
            self.channel.exchange_declare(exchange=self.exchange, exchange_type=self.exchange_type, durable=True)
            self.channel.queue_declare(queue=self.queue_name, durable=True)
            self.channel.basic_consume(
                queue=self.queue_name,  # 队列名
                on_message_callback=callback,  # 指定回调函数
                auto_ack=False,  # 关闭自动ack采用手动应答
            )
            try:
                self.channel.start_consuming()  # 开始接收信息，并进入阻塞状态，队列里有信息才会调用on_message_callback进行处理
            except Exception as e:
                print('consumer Error: ' + traceback.format_exc())
                self.connect.close()
                self.get_connect()
                pass

# --------------

def connected_to_spider_mq():
    return RabbitMQ(username=RabbitMQConfig.username,
                    password=RabbitMQConfig.password,
                    host=RabbitMQConfig.host,
                    port=RabbitMQConfig.port,
                    virtual_host=RabbitMQConfig.vhost,
                    exchange='spiderTask',
                    routing_key='#',
                    queue_name='spiderTask.groupName',
                    exchange_type='x-delayed-message',
                    arguments={'x-delayed-type': 'direct'}
                    )


def connected_to_spider_result_mq():
    return RabbitMQ(username=RabbitMQConfig.username,
                    password=RabbitMQConfig.password,
                    host=RabbitMQConfig.host,
                    port=RabbitMQConfig.port,
                    virtual_host=RabbitMQConfig.vhost,
                    exchange='spiderResult',
                    routing_key='#',
                    queue_name='spiderResult.groupName',
                    exchange_type='direct'
                    )


# 消费者消费时basic_consume函数中的on_message_callback参数会执行此方法，我们Python程序进而在这里可以拿到消息内容 和 给rabbitmq回调ack
def process_spider_task_msg(ch, method, properties, body):
    body_str = body.decode('utf-8')  # 在这里对消息内容进行我们想做的处理

    # headers = {'x-delay': 5000, 'x-delayed-type': 'direct'}
    # print("headers = " + str(properties.headers))

    body_json = json.loads(body_str)

    req_info = {
        handleManager.const_key_request_wrap: {
            handleManager.const_key_context: body_json['context'] if 'context' in body_json else None,
            handleManager.const_key_params: {"task": body_json['task'] if 'task' in body_json else None,
                                             "spider_result": body_json[
                                                 'spider_result'] if 'spider_result' in body_json else None,
                                             "handle_result": body_json[
                                                 'handle_result'] if 'handle_result' in body_json else None
                                             },
            'origin': body_json
        }
    }
    result = handleManager.handle('spider', json.dumps(req_info))
    if result[handleManager.const_key_result_success]:
        handler_result_msg = {
            'context': result[handleManager.const_key_context],
            'task': body_json['task'] if 'task' in body_json else None,
            'spiderResult': result[handleManager.const_key_result]
        }
        if not hasattr(thread_local_context,
                       'spider_result_queue') or not thread_local_context.spider_result_queue:
            spider_result_queue = connected_to_spider_result_mq()
            thread_local_context.__setattr__('spider_result_queue', spider_result_queue)
        
        # 先发spider结果消息，再答复 ack
        thread_local_context.spider_result_queue.publishMsg(json.dumps(handler_result_msg))
        ch.basic_ack(delivery_tag=method.delivery_tag)



def load_mock_msg(file_name):
    path = 'F:/codes/python/clientSpider/spider_result_handler/mockTask/' + file_name
    with open(path, 'r') as f:
        return json.loads(f.read())


repo = GitRepository(local_path, remote_path)
repo.pull()
print('pull git.repo=' + remote_path + ' TO local=' + local_path)


def pull_code_and_reload():
    now = calendar.timegm(time.gmtime())
    if now - repo.ts > 0:
        repo.ts = now
        git_info = repo.pull()
        if random.randint(0, 10) == 0 or not operator.contains(git_info, 'Already up to date'):
            print('prepare reload code cache for git update: ' + git_info)
            reload = False
            while not reload:
                try:
                    handleManager.load_functions(True)
                    reload = True
                    print('Reload code cache Success for git update.')
                except Exception as e:
                    print('[warn, retryable] reload code cache Error: ', e)


def schedule_thread_task():
    while True:
        schedule.run_pending()
        time.sleep(schedule.idle_seconds())


if __name__ == '__main__':

    # 开子线程 去监听消费消息
    for i in range(8):
        # 实例化的rabbbitmq对象
        spider_task_queue = connected_to_spider_mq()
        p = threading.Thread(target=spider_task_queue.doConsumer, args=(process_spider_task_msg,))
        p.setDaemon(True)
        p.start()

    # 定期更新git仓库
    schedule.every(5).seconds.do(pull_code_and_reload)
    schedule_thread = threading.Thread(target=schedule_thread_task)
    schedule_thread.setDaemon(True)
    schedule_thread.start()

    # 发消息
    # 将要发送的消息
    sys_platform = platform.platform().lower()
    if "windows" in sys_platform:
        testMsg = load_mock_msg('m3u8TsTask.json')
        testMsg = load_mock_msg('normalHtml.json')
        spider_task_queue = connected_to_spider_mq()

        for i in range(5):
            spider_task_queue.publishMsg(json.dumps(testMsg, ensure_ascii=False), delay_ms=2000)
            print('生产者发送第 {} 个消息成功 '.format(i + 1))
    print('\n -- 主程序不被阻塞，执行其它操作 --- \n')
    time.sleep(10000)
