import schedule
import calendar
import time
import os
from git.repo import Repo
from git.repo.fun import is_git_dir


class GitRepository(object):
    """
    git仓库管理
    """

    ts = 0

    def __init__(self, local_path, repo_url, branch='master'):
        self.local_path = local_path
        self.repo_url = repo_url
        self.repo = None
        self.initial(repo_url, branch)

    def initial(self, repo_url, branch):
        """
        初始化git仓库
        :param repo_url:
        :param branch:
        :return:
        """
        if not os.path.exists(self.local_path):
            os.makedirs(self.local_path)

        git_local_path = os.path.join(self.local_path, '.git')
        if not is_git_dir(git_local_path):
            self.repo = Repo.clone_from(repo_url, to_path=self.local_path, branch=branch)
        else:
            self.repo = Repo(self.local_path)

    def pull(self):
        """
        从线上拉最新代码
        :return:
        """
        return self.repo.git.pull()

    def branches(self):
        """
        获取所有分支
        :return:
        """
        branches = self.repo.remote().refs
        return [item.remote_head for item in branches if item.remote_head not in ['HEAD', ]]

    def commits(self):
        """
        获取所有提交记录
        :return:
        """
        commit_log = self.repo.git.log('--pretty={"commit":"%h","author":"%an","summary":"%s","date":"%cd"}',
                                       max_count=10,
                                       date='format:%Y-%m-%d %H:%M')
        log_list = commit_log.split("\n")
        # print(self.repo.heads.master.commit,type(self.repo.heads.master.commit),type(str(self.repo.heads.master.commit)))
        return [eval(item) for item in log_list]

    def new_master_commits(self):
        """获取主分支的最后一次提交记录"""
        return str(self.repo.heads.master.commit)[:7]

    def tags(self):
        """
        获取所有tag
        :return:
        """
        return [tag.name for tag in self.repo.tags]

    def change_to_branch(self, branch):
        """
        切换分值
        :param branch:
        :return:
        """
        self.repo.git.checkout(branch)

    def change_to_commit(self, branch, commit):
        """
        切换commit
        :param branch:
        :param commit:
        :return:
        """
        self.change_to_branch(branch=branch)
        self.repo.git.reset('--hard', commit)

    def change_to_tag(self, tag):
        """
        切换tag
        :param tag:
        :return:
        """
        self.repo.git.checkout(tag)

    def get_log_1(self):
        """获取当前版本信息"""
        commit_hash = self.repo.head.object.hexsha
        commit_message = self.repo.head.object.message
        commit_author = self.repo.head.object.author.name
        commit_time = self.repo.head.object.authored_datetime

        print(f"Commit hash: {commit_hash}")
        print(f"Commit message: {commit_message}")
        print(f"Commit author: {commit_author}")
        print(f"Commit time: {commit_time}")


# 本地仓库位置
local_path = os.environ.get('LOCAL_GIT_DIR', "F:/codes/python/clientSpider/testDir/git/") + 'testcicd'
# 远程仓库位置
remote_path = os.environ.get('REMOVE_GIT_PATH', 'http://git.itlym.cn/gogs/testcicd')

repo = GitRepository(local_path, remote_path)
print(repo.pull())


def pull_code():
    now = calendar.timegm(time.gmtime())
    if now - repo.ts > 0:
        repo.ts = now
        print(repo.pull())


if __name__ == '__main__':

    schedule.every(3).seconds.do(pull_code)
    while True:
        schedule.run_pending()
        time.sleep(schedule.idle_seconds())
